@@ -1,7 +1,8 @@
 /* -*- mode: C; c-file-style: "linux" -*- */
 
 /* MemProf -- memory profiler and leak detector
- * Copyright (C) 1999 Red Hat, Inc.
+ * Copyright 1999, 2000, 2001, Red Hat, Inc.
+ * Copyright 2002, Kristian Rietveld
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,6 +34,8 @@
 #include <glade/glade.h>
 #include <gnome.h>
 
+#include <gconf/gconf-client.h>
+
 #include <regex.h>
 
 #include "leakdetect.h"
@@ -80,29 +83,24 @@ struct _ProcessWindow {
 
 char *glade_file;
 
-#define DEFAULT_SKIP "g_malloc g_malloc0 g_realloc g_strdup strdup strndup"
-#define DEFAULT_SKIP_REGEXES ""
-#define DEFAULT_STACK_COMMAND "emacsclient -n +%l \"%f\""
-
 MPServer *global_server;
 
 static ProcessWindow *process_window_new (void);
 static void process_window_destroy (ProcessWindow *pwin);
 static void process_window_reset   (ProcessWindow *pwin);
 
-static int n_skip_funcs;
-static char **skip_funcs;
-
-static int n_skip_regexes;
-static char **skip_regexes;
+static GSList *skip_funcs = NULL;
+static GSList *skip_regexes = NULL;
 
 static gboolean default_follow_fork = FALSE;
 static gboolean default_follow_exec = FALSE;
 
-char *stack_command;
+char *stack_command = NULL;
 
 GSList *process_windows = NULL;
 
+static GladeXML *pref_dialog_xml = NULL; /* We save this around, so we can prevent multiple property boxes from being opened at once */
+
 
 /************************************************************
  * Status Page 
@@ -319,7 +317,7 @@ profile_ref_button_press (GtkWidget      *widget,
 			}
 		}
 
-		gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "button_press_event");
+		g_signal_stop_emission_by_name (G_OBJECT (widget), "button_press_event");
 		return TRUE;
 	}
 	
@@ -369,23 +367,24 @@ leak_block_select_row (GtkWidget     *widget,
 		       ProcessWindow *pwin)
 {
 	Block *block = gtk_clist_get_row_data (GTK_CLIST (widget), row);
-	int i;
+	StackElement *stack;
 
 	if (block == NULL)
 		return;
 
 	gtk_clist_clear (GTK_CLIST (pwin->leak_stack_clist));
-	for (i = 0; i < block->stack_size; i++) {
+	for (stack = block->stack; !STACK_ELEMENT_IS_ROOT (stack); stack = stack->parent) {
 		char *data[3];
 		char buf[32];
 		const char *filename;
 		char *functionname;
 		unsigned int line;
 		
-		if (!process_find_line (pwin->process, block->stack[i],
+		if (!process_find_line (pwin->process, stack->address,
 					&filename, &functionname, &line)) {
-			functionname = g_strdup ("(???)");
-			filename = "(???)";
+			/* 0x3f == '?' -- suppress trigraph warnings */
+			functionname = g_strdup ("(\x3f\x3f\x3f)");
+			filename = "(\x3f\x3f\x3f)";
 			line = 0;
 		}
 		
@@ -416,20 +415,20 @@ leaks_fill (ProcessWindow *pwin, GtkCList *clist)
 		const char *filename;
 		char *functionname = NULL;
 		unsigned int line;
-		int frame;
 		
 		Block *block = tmp_list->data;
+		StackElement *stack;
 
-		for (frame = 0 ; frame < block->stack_size ; frame++) {
-			if (process_find_line (pwin->process, block->stack[frame],
+		for (stack = block->stack; !STACK_ELEMENT_IS_ROOT (stack); stack = stack->parent) {
+			if (process_find_line (pwin->process, stack->address,
 					       &filename, &functionname, &line)) {
-				int i;
+				GSList *tmp_list;
 
 				if (!functionname)
 					continue;
 
-				for (i = 0; i < n_skip_funcs; i++) {
-					if (!strcmp (functionname, skip_funcs[i])) {
+				for (tmp_list = skip_funcs; tmp_list != NULL; tmp_list = tmp_list->next) {
+					if (!strcmp (functionname, tmp_list->data)) {
 						free (functionname);
 						functionname = NULL;
 						break;
@@ -439,10 +438,10 @@ leaks_fill (ProcessWindow *pwin, GtkCList *clist)
 				if (!functionname)
 					continue;
 
-				for (i = 0; i < n_skip_regexes; i++) {
+				for (tmp_list = skip_regexes; tmp_list != NULL; tmp_list = tmp_list->next) {
 					regex_t regex;
 
-					regcomp (&regex, skip_regexes[i], 0);
+					regcomp (&regex, tmp_list->data, 0);
 
 					if (!regexec (&regex, functionname, 0, NULL, 0)) {
 						free (functionname);
@@ -459,7 +458,7 @@ leaks_fill (ProcessWindow *pwin, GtkCList *clist)
 				break;
 		}
 		if (!functionname)
-			functionname = g_strdup ("(???)");
+			functionname = g_strdup ("(\x3f\x3f\x3f)");
 		
 		g_snprintf(buf, 32, "%p", block->addr);
 		data[0] = g_strdup (buf);
@@ -490,7 +489,11 @@ leak_stack_run_command (ProcessWindow *pwin, Block *block, int frame)
 	char *functionname;
 	unsigned int line;
 
-	if (process_find_line (pwin->process, block->stack[frame],
+	StackElement *stack = block->stack;
+	while (frame--)
+		stack = stack->parent;
+
+	if (process_find_line (pwin->process, stack->address,
 			       &filename, &functionname, &line)) {
 		
 		GString *command = g_string_new (NULL);
@@ -527,7 +530,8 @@ leak_stack_run_command (ProcessWindow *pwin, Block *block, int frame)
 		args[2] = command->str;
 
 		if (gnome_execute_async (NULL, 3, args) == -1) {
-			show_error (ERROR_MODAL, _("Executation of \"%s\" failed"),
+			show_error (pwin->main_window,
+				    ERROR_MODAL, _("Executation of \"%s\" failed"),
 				    command->str);
 		}
 
@@ -561,7 +565,7 @@ leak_stack_button_press (GtkWidget      *widget,
 			leak_stack_run_command (pwin, block, my_row);
 		}
 
-		gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "button_press_event");
+		g_signal_stop_emission_by_name (G_OBJECT (widget), "button_press_event");
 		return TRUE;
 	}
 	
@@ -650,15 +654,21 @@ void
 exit_cb (GtkWidget *widget)
 {
 	GtkWidget *dialog;
+	gint response;
 
 	ProcessWindow *pwin = pwin_from_widget (widget);
        
-	dialog = gnome_message_box_new (_("Really quit MemProf?"), GNOME_MESSAGE_BOX_QUESTION,
-					GNOME_STOCK_BUTTON_YES, GNOME_STOCK_BUTTON_NO, NULL);
+	dialog = gtk_message_dialog_new (GTK_WINDOW (pwin->main_window),
+					 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+					 GTK_MESSAGE_QUESTION,
+					 GTK_BUTTONS_YES_NO,
+					 _("Really quit MemProf?"));
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_YES);
 
-	gnome_dialog_set_parent (GNOME_DIALOG (dialog), GTK_WINDOW (pwin->main_window));
+	response = gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
 	
-	if (gnome_dialog_run (GNOME_DIALOG (dialog)) == 0)
+	if (response == GTK_RESPONSE_YES)
 		gtk_main_quit ();
 }
 
@@ -681,12 +691,9 @@ status_changed_cb (MPProcess *process, ProcessWindow *pwin)
 			tree_window_remove (pwin);
 		
 			if (pwin->process) {
-				gtk_signal_disconnect_by_func (GTK_OBJECT (pwin->process),
-							       GTK_SIGNAL_FUNC (status_changed_cb), pwin);
-				
-				gtk_signal_disconnect_by_func (GTK_OBJECT (pwin->process),
-							       GTK_SIGNAL_FUNC (reset_cb), pwin);
-				gtk_object_unref (GTK_OBJECT (pwin->process));
+				g_signal_handlers_disconnect_by_func (G_OBJECT (pwin->process), G_CALLBACK (status_changed_cb), pwin);
+				g_signal_handlers_disconnect_by_func (G_OBJECT (pwin->process), G_CALLBACK (reset_cb), pwin);
+				g_object_unref (G_OBJECT (pwin->process));
 				pwin->process = NULL;
 			}
 			
@@ -748,10 +755,10 @@ init_process (ProcessWindow *pwin, MPProcess *process)
 			       update_status,
 			       pwin);
 
-	gtk_signal_connect (GTK_OBJECT (process), "status_changed",
-			    GTK_SIGNAL_FUNC (status_changed_cb), pwin);
-	gtk_signal_connect (GTK_OBJECT (process), "reset",
-			    GTK_SIGNAL_FUNC (reset_cb), pwin);
+	g_signal_connect (G_OBJECT (process), "status_changed",
+			G_CALLBACK (status_changed_cb), pwin);
+	g_signal_connect (G_OBJECT (process), "reset",
+			G_CALLBACK (reset_cb), pwin);
 
 	tree_window_add (pwin);
 }
@@ -793,7 +800,8 @@ run_file (ProcessWindow *pwin, char **args)
 		result = TRUE;
 		
 	} else {
-		show_error (ERROR_MODAL,
+		show_error (pwin->main_window,
+			    ERROR_MODAL,
 			    _("Cannot find executable for \"%s\""),
 			    args[0]);
 		result = FALSE;
@@ -813,7 +821,7 @@ run_cb (GtkWidget *widget)
 
        ProcessWindow *pwin = pwin_from_widget (widget);
        
-       xml = glade_xml_new (glade_file, "RunDialog");
+       xml = glade_xml_new (glade_file, "RunDialog", NULL);
        run_dialog = glade_xml_get_widget (xml, "RunDialog");
        entry = glade_xml_get_widget (xml, "RunDialog-entry");
 
@@ -821,12 +829,12 @@ run_cb (GtkWidget *widget)
 				  GTK_SIGNAL_FUNC (gtk_widget_activate),
 				  GTK_OBJECT (glade_xml_get_widget (xml, "RunDialog-run")));
 
-       gtk_object_destroy (GTK_OBJECT (xml));
+       g_object_unref (G_OBJECT (xml));
 
        while (1) {
-	       gnome_dialog_set_parent (GNOME_DIALOG (run_dialog),
-					GTK_WINDOW (pwin->main_window));
-	       if (gnome_dialog_run (GNOME_DIALOG (run_dialog)) == 0) {
+	       gtk_window_set_transient_for (GTK_WINDOW (run_dialog),
+			                     GTK_WINDOW (pwin->main_window));
+	       if (gtk_dialog_run (GTK_DIALOG (run_dialog)) == 1) {
 		       gchar **args;
 		       char *text;
 		       gboolean result;
@@ -835,7 +843,7 @@ run_cb (GtkWidget *widget)
 		       args = process_parse_exec (text);
 
 		       result = run_file (pwin, args);
-		       
+
 		       g_strfreev (args);
 		       g_free (text);
 
@@ -969,7 +977,7 @@ generate_profile_cb (GtkWidget *widget)
 			pwin->profile = NULL;
 		}
 
-		pwin->profile = profile_create (pwin->process, skip_funcs, n_skip_funcs);
+		pwin->profile = profile_create (pwin->process, skip_funcs);
 		process_start_input (pwin->process);
 		profile_fill (pwin, GTK_CLIST (pwin->profile_func_clist));
 
@@ -977,70 +985,15 @@ generate_profile_cb (GtkWidget *widget)
 	}
 }
 
-static void
-preferences_apply_cb (GtkWidget *widget, gint page_num, GladeXML *preferences_xml)
-{
-	GtkWidget *skip_clist = glade_xml_get_widget (preferences_xml,
-						      "skip-clist");
-	GtkWidget *skip_regexes_clist = glade_xml_get_widget (preferences_xml,
-							     "skip-regexes-clist");
-	GtkWidget *stack_command_entry = glade_xml_get_widget (preferences_xml,
-							       "stack-command-entry");
-
-	gchar *text;
-	int i;
-
-	for (i=0; i < n_skip_funcs; i++)
-		g_free (skip_funcs[i]);
-	g_free (skip_funcs);
-
-	for (i=0; i < n_skip_regexes; i++)
-		g_free (skip_regexes[i]);
-	g_free (skip_regexes);
-
-	n_skip_funcs = GTK_CLIST (skip_clist)->rows;
-	skip_funcs = g_new (gchar *, n_skip_funcs);
-	
-	n_skip_regexes = GTK_CLIST (skip_regexes_clist)->rows;
-	skip_regexes = g_new (gchar *, n_skip_regexes);
-	
-	for (i=0; i < n_skip_funcs; i++) {
-		gchar *text;
-		gtk_clist_get_text (GTK_CLIST (skip_clist), i, 0, &text);
-		skip_funcs[i] = g_strdup (text);
-	}
-
-	for (i=0; i < n_skip_regexes; i++) {
-		gchar *text;
-		gtk_clist_get_text (GTK_CLIST (skip_regexes_clist), i, 0, &text);
-		skip_regexes[i] = g_strdup (text);
-	}
-
-	gnome_config_set_vector ("/MemProf/Options/skip_funcs",
-				 n_skip_funcs, (const char **const)skip_funcs);
-
-	gnome_config_set_vector ("/MemProf/Options/skip_regexes",
-				 n_skip_regexes, (const char **const)skip_regexes);
-
-	text = gtk_editable_get_chars (GTK_EDITABLE (stack_command_entry), 0, -1);
-	if (stack_command)
-		g_free (stack_command);
-	stack_command = text;
-	gnome_config_set_string ("/MemProf/Options/stack_command", text);
-	
-	gnome_config_sync ();
-}
-
 static void
 skip_fill (GtkWidget *clist)
 {
-       int i;
+	GSList *tmp_list;
 
-       gtk_clist_clear (GTK_CLIST (clist));
+	gtk_clist_clear (GTK_CLIST (clist));
 
-       for (i=0; i<n_skip_funcs; i++) {
-	       gtk_clist_append (GTK_CLIST (clist), &skip_funcs[i]);
-       }
+	for (tmp_list = skip_funcs; tmp_list != NULL; tmp_list = tmp_list->next)
+		gtk_clist_append (GTK_CLIST (clist), (gchar **)&tmp_list->data);
 }
 
 static void
@@ -1054,7 +1007,7 @@ skip_add_cb (GtkWidget *widget, GladeXML *preferences_xml)
        GtkWidget *skip_clist = glade_xml_get_widget (preferences_xml, "skip-clist");
        GtkWidget *property_box = glade_xml_get_widget (preferences_xml, "Preferences");
 
-       xml = glade_xml_new (glade_file, "SkipAddDialog");
+       xml = glade_xml_new (glade_file, "SkipAddDialog", NULL);
        dialog = glade_xml_get_widget (xml, "SkipAddDialog");
        entry = glade_xml_get_widget (xml, "SkipAddDialog-entry");
 
@@ -1062,14 +1015,15 @@ skip_add_cb (GtkWidget *widget, GladeXML *preferences_xml)
 				  GTK_SIGNAL_FUNC (gtk_widget_activate),
 				  GTK_OBJECT (glade_xml_get_widget (xml, "SkipAddDialog-add")));
 
-       gtk_object_destroy (GTK_OBJECT (xml));
+       g_object_unref (G_OBJECT (xml));
 
        while (1) {
-	       gnome_dialog_set_parent (GNOME_DIALOG (dialog),
-					GTK_WINDOW (property_box));
-	       if (gnome_dialog_run (GNOME_DIALOG (dialog)) == 0) {
+               gtk_window_set_transient_for (GTK_WINDOW (dialog),
+				             GTK_WINDOW (property_box));
+
+	       if (gtk_dialog_run (GTK_DIALOG (dialog)) == 1) {
 		       text = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
-		       
+
 		       if (strchr (text, ' ')) {
 			       GtkWidget *error = gnome_error_dialog_parented ("Function names cannot contain spaces",									       GTK_WINDOW (dialog));
 			       gnome_dialog_run (GNOME_DIALOG (error));
@@ -1081,64 +1035,64 @@ skip_add_cb (GtkWidget *widget, GladeXML *preferences_xml)
 			       g_free (text);
 		       } else {
 			       gtk_clist_append (GTK_CLIST (skip_clist), &text);
+
+			       skip_funcs = g_slist_append (skip_funcs,
+							    g_strdup (text));
+			       gconf_client_set_list (gconf_client_get_default (), "/apps/memprof/skip_funcs", GCONF_VALUE_STRING, skip_funcs, NULL);
+
 			       g_free (text);
-			       
-			       gnome_property_box_changed (GNOME_PROPERTY_BOX (property_box));
+
 			       break;
 		       }
-			       
 	       } else {
 		       break;
 	       }
        }
 
        gtk_widget_destroy (dialog);
-	
 }
 
 static void
 skip_delete_cb (GtkWidget *widget, GladeXML *preferences_xml)
 {
        GtkWidget *clist = glade_xml_get_widget (preferences_xml, "skip-clist");
-       GtkWidget *property_box = glade_xml_get_widget (preferences_xml, "Preferences");
 
 	if (GTK_CLIST (clist)->selection) {
+		GSList *tmp_list;
 		gint selected_row = GPOINTER_TO_INT (GTK_CLIST (clist)->selection->data);
 		gtk_clist_remove (GTK_CLIST (clist), selected_row);
 
-		gnome_property_box_changed (GNOME_PROPERTY_BOX (property_box));
+		tmp_list = g_slist_nth (skip_funcs, selected_row);
+		g_free (tmp_list->data);
+		skip_funcs = g_slist_delete_link (skip_funcs, tmp_list);
+
+		gconf_client_set_list (gconf_client_get_default (),
+				       "/apps/memprof/skip_funcs",
+				       GCONF_VALUE_STRING, skip_funcs, NULL);
 	}
 }
 
 static void
 skip_defaults_cb (GtkWidget *widget, GladeXML *preferences_xml)
 {
-       GtkWidget *clist = glade_xml_get_widget (preferences_xml, "skip-clist");
-       GtkWidget *property_box = glade_xml_get_widget (preferences_xml, "Preferences");
+       GConfClient *c = gconf_client_get_default ();
 
-	gchar **funcs = g_strsplit (DEFAULT_SKIP, " ", -1);
-	int i;
+       g_slist_free (skip_funcs);
+       skip_funcs = NULL;
 
-	gtk_clist_clear (GTK_CLIST (clist));
-	for (i=0; funcs[i]; i++) {
-	       gtk_clist_append (GTK_CLIST (clist), &funcs[i]);
-	}
-
-	g_strfreev (funcs);
-
-	gnome_property_box_changed (GNOME_PROPERTY_BOX (property_box));
+       /* we unset this here, notification will update it */
+       gconf_client_unset (c, "/apps/memprof/skip_funcs", NULL);
 }
 
 static void
 skip_regexes_fill (GtkWidget *clist)
 {
-       int i;
+	GSList *tmp_list;
 
-       gtk_clist_clear (GTK_CLIST (clist));
+	gtk_clist_clear (GTK_CLIST (clist));
 
-       for (i=0; i<n_skip_regexes; i++) {
-	       gtk_clist_append (GTK_CLIST (clist), &skip_regexes[i]);
-       }
+	for (tmp_list = skip_regexes; tmp_list != NULL; tmp_list = tmp_list->next)
+		gtk_clist_append (GTK_CLIST (clist), (gchar **)&tmp_list->data);
 }
 
 static void
@@ -1152,7 +1106,7 @@ skip_regexes_add_cb (GtkWidget *widget, GladeXML *preferences_xml)
        GtkWidget *skip_regexes_clist = glade_xml_get_widget (preferences_xml, "skip-regexes-clist");
        GtkWidget *property_box = glade_xml_get_widget (preferences_xml, "Preferences");
 
-       xml = glade_xml_new (glade_file, "SkipRegexesAddDialog");
+       xml = glade_xml_new (glade_file, "SkipRegexesAddDialog", NULL);
        dialog = glade_xml_get_widget (xml, "SkipRegexesAddDialog");
        entry = glade_xml_get_widget (xml, "SkipRegexesAddDialog-entry");
 
@@ -1160,12 +1114,13 @@ skip_regexes_add_cb (GtkWidget *widget, GladeXML *preferences_xml)
 				  GTK_SIGNAL_FUNC (gtk_widget_activate),
 				  GTK_OBJECT (glade_xml_get_widget (xml, "SkipRegexesAddDialog-add")));
 
-       gtk_object_destroy (GTK_OBJECT (xml));
+       g_object_unref (G_OBJECT (xml));
 
        while (1) {
-	       gnome_dialog_set_parent (GNOME_DIALOG (dialog),
-					GTK_WINDOW (property_box));
-	       if (gnome_dialog_run (GNOME_DIALOG (dialog)) == 0) {
+               gtk_window_set_transient_for (GTK_WINDOW (dialog),
+                                             GTK_WINDOW (property_box));
+
+	       if (gtk_dialog_run (GTK_DIALOG (dialog)) == 1) {
 		       text = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
 		       
 		       if (strchr (text, ' ')) {
@@ -1179,128 +1134,213 @@ skip_regexes_add_cb (GtkWidget *widget, GladeXML *preferences_xml)
 			       g_free (text);
 		       } else {
 			       gtk_clist_append (GTK_CLIST (skip_regexes_clist), &text);
+
+			       skip_regexes = g_slist_append (skip_regexes,
+							      g_strdup (text));
+			       gconf_client_set_list (gconf_client_get_default (), "/apps/memprof/skip_regexes", GCONF_VALUE_STRING, skip_regexes, NULL);
+
 			       g_free (text);
-			       
-			       gnome_property_box_changed (GNOME_PROPERTY_BOX (property_box));
+
 			       break;
 		       }
-			       
 	       } else {
 		       break;
 	       }
        }
 
        gtk_widget_destroy (dialog);
-	
 }
 
 static void
 skip_regexes_delete_cb (GtkWidget *widget, GladeXML *preferences_xml)
 {
        GtkWidget *clist = glade_xml_get_widget (preferences_xml, "skip-regexes-clist");
-       GtkWidget *property_box = glade_xml_get_widget (preferences_xml, "Preferences");
 
 	if (GTK_CLIST (clist)->selection) {
+		GSList *tmp_list;
 		gint selected_row = GPOINTER_TO_INT (GTK_CLIST (clist)->selection->data);
 		gtk_clist_remove (GTK_CLIST (clist), selected_row);
 
-		gnome_property_box_changed (GNOME_PROPERTY_BOX (property_box));
+		tmp_list = g_slist_nth (skip_regexes, selected_row);
+		g_free (tmp_list->data);
+		skip_regexes = g_slist_delete_link (skip_regexes, tmp_list);
+
+		gconf_client_set_list (gconf_client_get_default (),
+				       "/apps/memprof/skip_regexes",
+				       GCONF_VALUE_STRING, skip_regexes, NULL);
 	}
 }
 
 static void
 skip_regexes_defaults_cb (GtkWidget *widget, GladeXML *preferences_xml)
 {
-       GtkWidget *clist = glade_xml_get_widget (preferences_xml, "skip-regxes-clist");
-       GtkWidget *property_box = glade_xml_get_widget (preferences_xml, "Preferences");
+       GConfClient *c = gconf_client_get_default ();
 
-	gchar **funcs = g_strsplit (DEFAULT_SKIP_REGEXES, " ", -1);
-	int i;
+       g_slist_free (skip_regexes);
+       skip_regexes = NULL;
 
-	gtk_clist_clear (GTK_CLIST (clist));
-	for (i=0; funcs[i]; i++) {
-	       gtk_clist_append (GTK_CLIST (clist), &funcs[i]);
-	}
+       /* we unset this there, notification will update it */
+       gconf_client_unset (c, "/apps/memprof/skip_regexes", NULL);
+}
 
-	g_strfreev (funcs);
+static void
+stack_command_entry_update (GtkWidget *entry,
+			    gpointer data)
+{
+	const gchar *new_value;
 
-	gnome_property_box_changed (GNOME_PROPERTY_BOX (property_box));
+	new_value = gtk_entry_get_text (GTK_ENTRY (entry));
+
+	if (new_value)
+		gconf_client_set_string (gconf_client_get_default (),
+					 "/apps/memprof/stack_command",
+					 new_value, NULL);
+}
+
+static void
+stack_command_entry_focus_out (GtkWidget *entry,
+			       GdkEventFocus *ev,
+			       gpointer data)
+{
+	stack_command_entry_update (entry, NULL);
 }
 
 void
-preferences_destroy_cb (GtkWidget *widget, GtkObject **xml)
+preferences_destroy_cb (GtkWidget *widget,
+			GObject **xml)
 {
 	g_assert( xml != NULL );
 
-	gtk_object_destroy(*xml);
+	g_object_unref(*xml);
 	/* Mark the property box as destroyed so we don't try to touch it again */
 	*xml = NULL;
 }
 
+void
+preferences_response_cb (GtkWidget *w, int response, gpointer data)
+{
+	if (response == GTK_RESPONSE_CLOSE)
+		gtk_widget_destroy (w);
+}
+
 void
 preferences_cb (GtkWidget *widget)
 {
-       static GladeXML *xml = NULL; /* Static so we can prevent multiple property boxes from being opened at once */
-       GtkWidget *skip_clist, *skip_regexes_clist;
        static GtkWidget *property_box = NULL;
+       GtkWidget *skip_clist, *skip_regexes_clist;
        GtkWidget *stack_command_entry;
        GtkWidget *button;
+       GladeXML *xml;
 
        /* Check for an open property box */
-       if (xml != NULL)
+       if (pref_dialog_xml != NULL)
        {
 		gdk_window_show( GTK_WIDGET(property_box)->window );
 		gdk_window_raise( GTK_WIDGET(property_box)->window );
 		return;
        }
 
-       xml = glade_xml_new (glade_file, "Preferences");
+       pref_dialog_xml = glade_xml_new (glade_file, "Preferences", NULL);
+       xml = pref_dialog_xml;
        skip_clist = glade_xml_get_widget (xml, "skip-clist");
        skip_regexes_clist = glade_xml_get_widget (xml, "skip-regexes-clist");
        stack_command_entry = glade_xml_get_widget (xml, "stack-command-entry");
-       property_box = glade_xml_get_widget (xml, "Preferences");
+
+       property_box = gtk_dialog_new_with_buttons ("Preferences",
+						   NULL,
+						   GTK_DIALOG_DESTROY_WITH_PARENT,
+						   GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+						   NULL);
+       gtk_widget_reparent (glade_xml_get_widget (xml, "notebook2"),
+                            GTK_DIALOG (property_box)->vbox);
 
        skip_fill (skip_clist);
        skip_regexes_fill (skip_regexes_clist);
 
        gtk_entry_set_text (GTK_ENTRY (stack_command_entry), stack_command);
-       gtk_signal_connect_object (GTK_OBJECT (stack_command_entry),
-				  "changed",
-				  GTK_SIGNAL_FUNC (gnome_property_box_changed),
-				  GTK_OBJECT (property_box));
-       
+       g_signal_connect (G_OBJECT (stack_command_entry), "activate",
+			 G_CALLBACK (stack_command_entry_update), NULL);
+       g_signal_connect (G_OBJECT (stack_command_entry), "focus_out_event",
+			 G_CALLBACK (stack_command_entry_focus_out), NULL);
+
        button = glade_xml_get_widget (xml, "skip-add-button");
-       gtk_signal_connect (GTK_OBJECT (button), "clicked",
-			   GTK_SIGNAL_FUNC (skip_add_cb), xml);
-
+       g_signal_connect (G_OBJECT (button), "clicked",
+			 G_CALLBACK (skip_add_cb), xml);
        button = glade_xml_get_widget (xml, "skip-delete-button");
-       gtk_signal_connect (GTK_OBJECT (button), "clicked",
-			   GTK_SIGNAL_FUNC (skip_delete_cb), xml);
-
+       g_signal_connect (G_OBJECT (button), "clicked",
+			 G_CALLBACK (skip_delete_cb), xml);
        button = glade_xml_get_widget (xml, "skip-defaults-button");
-       gtk_signal_connect (GTK_OBJECT (button), "clicked",
-			   GTK_SIGNAL_FUNC (skip_defaults_cb), xml);
+       g_signal_connect (G_OBJECT (button), "clicked",
+			 G_CALLBACK (skip_defaults_cb), xml);
 
        button = glade_xml_get_widget (xml, "skip-regexes-add-button");
-       gtk_signal_connect (GTK_OBJECT (button), "clicked",
-			   GTK_SIGNAL_FUNC (skip_regexes_add_cb), xml);
-
+       g_signal_connect (G_OBJECT (button), "clicked",
+			 G_CALLBACK (skip_regexes_add_cb), xml);
        button = glade_xml_get_widget (xml, "skip-regexes-delete-button");
-       gtk_signal_connect (GTK_OBJECT (button), "clicked",
-			   GTK_SIGNAL_FUNC (skip_regexes_delete_cb), xml);
-
+       g_signal_connect (G_OBJECT (button), "clicked",
+			 G_CALLBACK (skip_regexes_delete_cb), xml);
        button = glade_xml_get_widget (xml, "skip-regexes-defaults-button");
-       gtk_signal_connect (GTK_OBJECT (button), "clicked",
-			   GTK_SIGNAL_FUNC (skip_regexes_defaults_cb), xml);
+       g_signal_connect (G_OBJECT (button), "clicked",
+			 G_CALLBACK (skip_regexes_defaults_cb), xml);
 
        glade_xml_signal_autoconnect (xml);
 
-       gtk_signal_connect (GTK_OBJECT (property_box), "apply",
-			   GTK_SIGNAL_FUNC (preferences_apply_cb),
-			   GTK_OBJECT (xml));
-       gtk_signal_connect (GTK_OBJECT (property_box), "destroy",
-		       	   GTK_SIGNAL_FUNC(preferences_destroy_cb),
-			   &(xml));
+       g_signal_connect (G_OBJECT (property_box), "destroy",
+			 G_CALLBACK (preferences_destroy_cb), &(pref_dialog_xml));
+       g_signal_connect (G_OBJECT (property_box), "response",
+			 G_CALLBACK (preferences_response_cb), NULL);
+
+       gtk_widget_show_all (property_box);
+}
+
+static void
+preferences_list_notify (GConfClient *client,
+			 guint cnx_id,
+			 GConfEntry *entry,
+			 gpointer user_data)
+{
+	if (!strcmp(gconf_entry_get_key (entry), "/apps/memprof/skip_funcs")) {
+		if (skip_funcs)
+			g_slist_free (skip_funcs);
+		skip_funcs = gconf_client_get_list (client,
+						    "/apps/memprof/skip_funcs",
+						    GCONF_VALUE_STRING, NULL);
+
+		if (pref_dialog_xml) {
+			GtkWidget *clist = glade_xml_get_widget (pref_dialog_xml, "skip-clist");
+			skip_fill (clist);
+		}
+	} else if (!strcmp (gconf_entry_get_key (entry), "/apps/memprof/skip_regexes")) {
+		if (skip_regexes)
+			g_slist_free (skip_regexes);
+		skip_regexes = gconf_client_get_list (client, "/apps/memprof/skip_regexes",
+						      GCONF_VALUE_STRING, NULL);
+
+		if (pref_dialog_xml) {
+			GtkWidget *clist = glade_xml_get_widget (pref_dialog_xml, "skip-regexes-clist");
+			skip_regexes_fill (clist);
+		}
+	}
+}
+
+static void
+preferences_stack_command_notify (GConfClient *client,
+				  guint cnx_id,
+				  GConfEntry *entry,
+				  gpointer user_data)
+{
+	if (stack_command)
+		g_free (stack_command);
+	stack_command = gconf_client_get_string (client,
+						 "/apps/memprof/stack_command",
+						 NULL);
+
+	if (pref_dialog_xml && stack_command) {
+		GtkWidget *e;
+
+		e = glade_xml_get_widget (pref_dialog_xml, "stack-command-entry");
+		gtk_entry_set_text (GTK_ENTRY (e), stack_command);
+	}
 }
 
 void
@@ -1329,50 +1369,59 @@ about_cb (GtkWidget *widget)
 
        ProcessWindow *pwin = pwin_from_widget (widget);
 	
-       xml = glade_xml_new (glade_file, "About");
+       xml = glade_xml_new (glade_file, "About", NULL);
        dialog = glade_xml_get_widget (xml, "About");
-       gtk_object_destroy (GTK_OBJECT (xml));
+       g_object_unref (G_OBJECT (xml));
 
-       gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
-       gnome_dialog_set_parent (GNOME_DIALOG (dialog),
-				GTK_WINDOW (pwin->main_window));
+       gtk_window_set_transient_for (GTK_WINDOW (dialog),
+                                     GTK_WINDOW (pwin->main_window));
+       g_object_set (G_OBJECT (dialog), "version", VERSION, NULL);
 
        gtk_widget_show (dialog);
 }
 
-void 
-show_error (ErrorType error,
+static void
+show_error_response (GtkDialog *dialog,
+		     gint       response_id,
+		     gpointer   user_data)
+{
+	if (response_id == GTK_RESPONSE_OK)
+		gtk_widget_destroy (GTK_WIDGET (dialog));
+}
+
+void
+show_error (GtkWidget *parent_window,
+	    ErrorType error,
 	    const gchar *format,
 	    ...)
 {
 	va_list args;
 	char *message;
 	GtkWidget *dialog;
-	
+
 	va_start (args, format);
 	vasprintf (&message, format, args);
 	va_end (args);
 
-	dialog = gnome_message_box_new (message,
-					(error == ERROR_WARNING) ?
-					  GNOME_MESSAGE_BOX_WARNING :
-					  GNOME_MESSAGE_BOX_ERROR,
-					GNOME_STOCK_BUTTON_OK, NULL);
+	dialog = gtk_message_dialog_new (parent_window ? GTK_WINDOW (parent_window) : NULL,
+					 GTK_DIALOG_DESTROY_WITH_PARENT,
+					 (error == ERROR_FATAL) ?
+					   GTK_MESSAGE_ERROR :
+					   GTK_MESSAGE_WARNING,
+					 GTK_BUTTONS_OK, message);
 	free (message);
 
 	gtk_window_set_title (GTK_WINDOW (dialog),
-			      (error == ERROR_WARNING) ?
+			      (error == ERROR_FATAL) ?
 			      _("MemProf Error") : _("MemProf Warning"));
 
-	if (error == ERROR_WARNING)
+	if (error == ERROR_WARNING) {
 		gtk_widget_show (dialog);
-	else {
-#if 0
-		if (pwin->main_window)
-			gnome_dialog_set_parent (GNOME_DIALOG (dialog),
-						 GTK_WINDOW (pwin->main_window));
-#endif		
-		gnome_dialog_run (GNOME_DIALOG (dialog));
+		g_signal_connect (dialog, "response",
+				  G_CALLBACK (show_error_response), NULL);
+	} else {
+		gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
 	}
 
 	if (error == ERROR_FATAL)
@@ -1424,6 +1473,7 @@ process_window_destroy (ProcessWindow *pwin)
 static ProcessWindow *
 process_window_new (void)
 {
+       gchar *fullfilename;
        GladeXML *xml;
        GtkWidget *vpaned;
        ProcessWindow *pwin;
@@ -1440,9 +1490,13 @@ process_window_new (void)
        pwin->usage_high = 0;
        pwin->usage_leaked = 0;
 
-       xml = glade_xml_new (glade_file, "MainWindow");
+       xml = glade_xml_new (glade_file, "MainWindow", NULL);
 
        pwin->main_window = glade_xml_get_widget (xml, "MainWindow");
+       fullfilename = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "memprof.png", FALSE, NULL);
+       gnome_window_icon_set_from_file (GTK_WINDOW (pwin->main_window),
+					fullfilename);
+       g_free (fullfilename);
 
        gtk_signal_connect (GTK_OBJECT (pwin->main_window), "delete_event",
 			   GTK_SIGNAL_FUNC (hide_and_check_quit), pwin);
@@ -1540,7 +1594,7 @@ process_window_new (void)
        gtk_paned_set_position (GTK_PANED (vpaned), 150);
 
        glade_xml_signal_autoconnect (xml);
-       gtk_object_destroy (GTK_OBJECT (xml));
+       g_object_unref (G_OBJECT (xml));
 
        return pwin;
 }
@@ -1577,15 +1631,17 @@ process_window_hide (ProcessWindow *pwin)
 void
 check_quit (void)
 {
-	GList *toplevels;
+	GList *toplevels, *tmplist;
 	
-	toplevels = gtk_container_get_toplevels ();
-	while (toplevels) {
+	tmplist = toplevels = gtk_window_list_toplevels ();
+	while (tmplist) {
 		if (GTK_WIDGET_VISIBLE (toplevels->data))
 			return;
-		toplevels = toplevels->next;
+		tmplist = tmplist->next;
 	}
 
+	g_list_free (toplevels);
+
 	gtk_main_quit ();
 }
 
@@ -1604,18 +1660,24 @@ process_window_maybe_detach (ProcessWindow *pwin)
 {
 	GtkWidget *dialog;
 	const char *message;
+	gint response;
 
 	if (pwin->process->status == MP_PROCESS_EXITING)
 		message = _("Really detach from finished process?");
 	else
 		message = _("Really detach from running process?");
 
-	
-	dialog = gnome_message_box_new (message, GNOME_MESSAGE_BOX_QUESTION,
-					GNOME_STOCK_BUTTON_YES, GNOME_STOCK_BUTTON_NO, NULL);
-	
-	gnome_dialog_set_parent (GNOME_DIALOG (dialog), GTK_WINDOW (pwin->main_window));
-	if (gnome_dialog_run (GNOME_DIALOG (dialog)) == 0)
+	dialog = gtk_message_dialog_new (GTK_WINDOW (pwin->main_window),
+					 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+					 GTK_MESSAGE_QUESTION,
+					 GTK_BUTTONS_YES_NO,
+					 message);
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_YES);
+
+	response = gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+
+	if (response == GTK_RESPONSE_YES)
 		process_detach (pwin->process);
 }
 
@@ -1627,12 +1689,20 @@ process_window_maybe_kill (ProcessWindow *pwin)
 		process_window_maybe_detach (pwin);
 	else {
 		GtkWidget *dialog;
-		
-		dialog = gnome_message_box_new (_("Really kill running process?"), GNOME_MESSAGE_BOX_QUESTION,
-						GNOME_STOCK_BUTTON_YES, GNOME_STOCK_BUTTON_NO, NULL);
-		
-		gnome_dialog_set_parent (GNOME_DIALOG (dialog), GTK_WINDOW (pwin->main_window));
-		if (gnome_dialog_run (GNOME_DIALOG (dialog)) == 0)
+		gint response;
+
+		dialog = gtk_message_dialog_new (GTK_WINDOW (pwin->main_window),
+						 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+						 GTK_MESSAGE_QUESTION,
+						 GTK_BUTTONS_YES_NO,
+						 _("Really kill running process?"));
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog),
+						 GTK_RESPONSE_YES);
+
+		response = gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+
+		if (response == GTK_RESPONSE_YES)
 			process_kill (pwin->process);
 	}
 }
@@ -1665,20 +1735,31 @@ main(int argc, char **argv)
        };
        poptContext ctx;
 
-       int init_results;
+       GnomeProgram *program;
        const char **startup_args;
        ProcessWindow *initial_window;
 
        /* Set up a handler for SIGCHLD to avoid zombie children
 	*/
        signal (SIGCHLD, sigchld_handler);
-       
-       bindtextdomain (PACKAGE, GNOMELOCALEDIR);
-       textdomain (PACKAGE);
 
-       init_results = gnome_init_with_popt_table (PACKAGE, VERSION,
-						  argc, argv,
-						  memprof_popt_options, 0, &ctx);
+       bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+       textdomain (GETTEXT_PACKAGE);
+
+#ifdef HAVE_BIND_TEXTDOMAIN_CODESET
+       bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+#endif
+
+       program = gnome_program_init (PACKAGE, VERSION,
+				     LIBGNOMEUI_MODULE,
+				     argc, argv,
+				     GNOME_PARAM_POPT_TABLE, memprof_popt_options,
+				     NULL);
+
+       g_object_get (G_OBJECT (program),
+		     GNOME_PARAM_POPT_CONTEXT, &ctx,
+		     NULL);
+
        glade_gnome_init ();
 
        glade_file = "./memprof.glade";
@@ -1686,20 +1767,39 @@ main(int argc, char **argv)
 	       glade_file = g_concat_dir_and_file (DATADIR, "memprof.glade");
        }
        if (!g_file_exists (glade_file)) {
-	       show_error (ERROR_FATAL, _("Cannot find memprof.glade"));
+	       show_error (NULL, ERROR_FATAL, _("Cannot find memprof.glade"));
        }
 
        global_server = mp_server_new ();
-       gtk_signal_connect (GTK_OBJECT (global_server), "process_created",
-			   GTK_SIGNAL_FUNC (process_created_cb), NULL);
+       g_signal_connect (G_OBJECT (global_server), "process_created",
+		       G_CALLBACK (process_created_cb), NULL);
        
        initial_window = process_window_new ();
 
-       gnome_config_get_vector ("/MemProf/Options/skip_funcs=" DEFAULT_SKIP,
-				&n_skip_funcs, &skip_funcs);
-       gnome_config_get_vector ("/MemProf/Options/skip_regexes=" DEFAULT_SKIP_REGEXES,
-				&n_skip_regexes, &skip_regexes);
-       stack_command = gnome_config_get_string ("/MemProf/Options/stack_command=" DEFAULT_STACK_COMMAND);
+       gconf_client_add_dir (gconf_client_get_default (),
+		             "/apps/memprof",
+			     GCONF_CLIENT_PRELOAD_NONE, NULL);
+       skip_funcs = gconf_client_get_list (gconf_client_get_default (),
+					   "/apps/memprof/skip_funcs",
+					   GCONF_VALUE_STRING, NULL);
+       skip_regexes = gconf_client_get_list (gconf_client_get_default (),
+					     "/apps/memprof/skip_regexes",
+					     GCONF_VALUE_STRING, NULL);
+       stack_command = gconf_client_get_string (gconf_client_get_default (),
+						"/apps/memprof/stack_command",
+						NULL);
+       gconf_client_notify_add (gconf_client_get_default (),
+			        "/apps/memprof/stack_command",
+			        preferences_stack_command_notify,
+			        NULL, NULL, NULL);
+       gconf_client_notify_add (gconf_client_get_default (),
+				"/apps/memprof/skip_funcs",
+				preferences_list_notify,
+				NULL, NULL, NULL);
+       gconf_client_notify_add (gconf_client_get_default (),
+				"/apps/memprof/skip_regexes",
+				preferences_list_notify,
+				NULL, NULL, NULL);
 
        gtk_widget_show (initial_window->main_window);
 
@@ -1707,9 +1807,8 @@ main(int argc, char **argv)
        if (startup_args)
 	       run_file (initial_window, (char **)startup_args);
        poptFreeContext (ctx);
-       
+
        gtk_main ();
 
        return 0;
 }
-
