@@ -1,7 +1,8 @@
 /* -*- mode: C; c-file-style: "linux" -*- */
 
 /* MemProf -- memory profiler and leak detector
- * Copyright (C) 1999 Red Hat, Inc.
+ * Copyright 1999, 2000, 2001, Red Hat, Inc.
+ * Copyright 2002, Kristian Rietveld
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,7 +46,7 @@ typedef struct {
   GHashTable *skip_hash;
 } Profile;
 
-Profile *profile_create (MPProcess *process, char **skip_funcs, gint n_skip_funcs);
-void profile_write (Profile *profile, gchar *outfile);
+Profile *profile_create (MPProcess *process, GSList *skip_funcs);
+void profile_write (Profile *profile,
+		    gchar *outfile);
 void profile_free (Profile *profile);
-
